%define name MonetDB
%define release 1
%define version 4.8.2

%define prefix /usr
%define sysconfdir /etc
%define localstatedir /var/lib

%define monetdb_uid    85
%define monetdb_gid    85

Name: %{name}
Version: %{version}
Release: %{release}
Summary: MonetDB - Monet Database Management System
Group: System
Source: %{name}-%{version}.tar.gz
BuildRoot: /var/tmp/%{name}-root
Packager: Niels Nes <Niels.Nes@cwi.nl>
License:   MPL - http://monetdb.cwi.nl/Legal/MonetDBLicense-1.0.html
BuildRequires: python 

%define builddoc 0
%if 0
%define buildphp 1
%define buildperl 1
%define buildpython 1
%define comp_cc gcc
%define comp_cxx g++
%endif

# groups of related archs
%define all_x86 i386 i586 i686

%ifarch %{all_x86}
%define bits 32
%else
%define bits 64
%endif

%package client
Summary: MonetDB clients
Group: System

%if %{buildphp}
%package php
Summary: MonetDB php interface
Group: System
BuildRequires: php 
%define with_php --with-php
%else
%define with_php --without-php
%endif

%if %{buildpython}
%package python
Summary: MonetDB python interface
Group: System
BuildRequires: python
%endif

%if %{buildperl}
%package perl
Summary: MonetDB perl interface
Group: System
BuildRequires: perl 
%define with_perl --with-perl
%else
%define with_perl --without-perl
%endif

%package server
Summary: MonetDB server 
Group: System 

%package devel
Summary: MonetDB development package 
Group: System 

%if %{builddoc}
%package docu
Summary: MonetDB documentation
Group: System 
%endif

%description
MonetDB is a database management system that is developed from a
main-memory perspective with use of a fully decomposed storage model,
automatic index management, extensibility of data types and search
accellerators, SQL- and XML- frontends.

%description client
Add the MonetDB client description here

%if %{buildphp}
%description php
Add the MonetDB php description here
Requires: %{name}-client
Requires: php
%define php_extensiondir %(php-config --extension-dir)
%define php_peardir %(pear config-get php_dir | sed -e s+php_dir=++g)
%endif

%if %{buildpython}
%description python
Add the MonetDB python description here
Requires: %{name}-client
Requires: python
%define python_libdir %(python -c 'import distutils.sysconfig; print distutils.sysconfig.get_python_lib(1,0,"")')
%endif

%if %{buildperl}
%description perl
Add the MonetDB perl description here
Requires: %{name}-client
Requires: perl
%define perl_libdir %(perl -MConfig -e '$x=$Config{installvendorarch}; $x =~ s|$Config{vendorprefix}/||; print $x;')
%endif

%description server
Add the MonetDB server description here
Requires: %{name}-client

%description devel
Add the MonetDB devel description here
Requires: %{name}-server
Requires: epsffit
PreReq: /sbin/chkconfig, /sbin/service, sh-utils
PreReq: %{_sbindir}/groupadd, %{_sbindir}/useradd

%if %{builddoc}
%description docu
Add the MonetDB documentation description here
%endif


%prep
rm -rf $RPM_BUILD_ROOT

%setup -q -n %{name}-%{version}

%build

./configure \
	--enable-bits=%{bits} \
	--with-gcc="%{comp_cc}" \
	--with-gxx="%{comp_cxx}" \
	--prefix=%{prefix} \
	--sysconfdir=%{sysconfdir} \
	--localstatedir=%{localstatedir} \
	%{with_php} %{with_perl}

make

%install
rm -rf $RPM_BUILD_ROOT

make install \
	DESTDIR=$RPM_BUILD_ROOT 

mkdir -p $RPM_BUILD_ROOT/%{localstatedir}/MonetDB/dbfarm
# insert example db here!

find $RPM_BUILD_ROOT -name .incs.in | xargs rm -f

# cleanup stuff we don't want to install
rm -rf $RPM_BUILD_ROOT%{prefix}/bin/epsffit
rm -rf $RPM_BUILD_ROOT%{prefix}/share/%{name}/conf/conf.bash
rm -rf $RPM_BUILD_ROOT%{prefix}/share/%{name}/general.mil
rm -rf $RPM_BUILD_ROOT%{prefix}/share/%{name}/lady.gif
rm -rf $RPM_BUILD_ROOT%{prefix}/share/%{name}/docs/*/*.tex
rm -rf $RPM_BUILD_ROOT%{prefix}/share/%{name}/docs/*/*/*.tex

# Fixes monet config script
#perl -p -i -e "s|$RPM_BUILD_ROOT||" $RPM_BUILD_ROOT%{prefix}/bin/monet_config

%clean
rm -fr $RPM_BUILD_ROOT

%files client
%defattr(-,monetdb,monetdb) 
%{prefix}/bin/MapiClient* 
%{prefix}/lib/libmutils.*
%{prefix}/lib/libMapi.* 
%{prefix}/lib/libstream.*
%{prefix}/share/%{name}/lib/*jar
%{prefix}/lib/%{name}/Tests/*
%{prefix}/share/%{name}/Tests/*.class
%{prefix}/share/%{name}/python/* 

%config(noreplace) %{sysconfdir}/MonetDB.conf 

%pre client 
%{_sbindir}/groupadd -g %{monetdb_gid} -r monetdb 2>/dev/null || :
%{_sbindir}/useradd -d %{localstatedir}/%{name} -s /bin/true -g monetdb -M -r -u %{monetdb_uid} monetdb 2>/dev/null || :

%if %{buildphp}
%files php
%defattr(-,monetdb,monetdb) 
%{php_extensiondir}/*
%{php_peardir}/DB/*
%{prefix}/share/%{name}/php/*
%endif

%if %{buildpython}
%files python
%defattr(-,monetdb,monetdb) 
%{prefix}/%{python_libdir}/*
%endif

%if %{buildperl}
%files perl
%defattr(-,monetdb,monetdb) 
%{prefix}/%{perl_libdir}/*
%{prefix}/share/%{name}/perl/* 
%endif

%files server
%defattr(-,monetdb,monetdb) 
%{prefix}/bin/Mserver 

%{prefix}/lib/libbat.*
%{prefix}/lib/libmonet.*
%{prefix}/lib/%{name}/*.so* 
%{prefix}/lib/%{name}/*.la* 
%{prefix}/lib/%{name}/*.mil* 

%{prefix}/share/%{name}/tools/* 

%dir %{localstatedir}/%{name}

%files devel
%defattr(-,monetdb,monetdb) 
%{prefix}/bin/monetdb-config*
%{prefix}/share/%{name}/conf/monet.m4 

%{prefix}/bin/mel
%{prefix}/bin/calibrator
%{prefix}/bin/Mx 
%{prefix}/bin/prefixMxFile 
%{prefix}/bin/idxmx 

%{prefix}/share/%{name}/Mprofile-commands.lst 
%{prefix}/share/%{name}/quit.mil 

%{prefix}/bin/prof.py*
%{prefix}/bin/Mtest.py*
%{prefix}/bin/Mapprove.py*
%{prefix}/bin/Mfilter.py*
%{prefix}/bin/Mprofile.py*
%{prefix}/bin/Mlog*
%{prefix}/bin/Mdiff*
%{prefix}/bin/Mtimeout*
%{prefix}/bin/MkillUsers*

%{prefix}/lib/autogen/* 
%{prefix}/lib/pkgconfig/MonetDB.pc

%{prefix}/include/%{name}/*.h
%{prefix}/include/%{name}/*/*.[hcm]

%if %{builddoc}
%files docu
%defattr(-,monetdb,monetdb)
%{prefix}/share/%{name}/docs/*/*.html
%{prefix}/share/%{name}/docs/*/*.pdf
%{prefix}/share/%{name}/docs/*/*.ps
%{prefix}/share/%{name}/docs/*/*/*.html
%{prefix}/share/%{name}/docs/*/*/*.pdf
%{prefix}/share/%{name}/docs/*/*/*.ps
%endif

%post server
umask 022

#create monetdb init script
#/sbin/chkconfig --add monetdb

